/***************************************************************************
 * <p>Title: Presence Detection</p>
 * <p>Description: Projects In Computer Science</p>
 * <p>Copyright: Copyright (c) 2007</p>
 * <p>Company: Columbia University; all rights reserved</p>
 *
 * @author Denis Abramov (daa82@columbia.edu)
 * @version 1.0
 *
 * Description:
 * This is an algo that will try to find the average background
 * and subtract the average background from the image
 * to eliminate the background noise that is often associated
 * with image detection
 *
 ***************************************************************************/

package com.presencedetection.algos;

import java.awt.image.BufferedImage;
import java.util.Arrays;
import java.awt.Color;

public class BackgroundSubtractionAlgo
    implements PDAlgoIntf {

  private static final int BACKGROUND_THRESHOLD = 70;

  public BackgroundSubtractionAlgo() {}

  /**
   * analyze
   *
   * @param image BufferedImage
   * @return PDResults
   */
  public PDResults analyze(BufferedImage image) {
    BufferedImage i = removeBackgroundColor(image, true);
   ImageData im = new ImageData(i);
  return new PDResults(im.isPresenceDetected(20), 1, im);

  }

  /**
   * isRunning
   *
   * @return boolean
   */
  public boolean isRunning() {
    return true;
  }

  /**
   * start
   *
   * @return boolean
   */
  public boolean start() {
    return false;
  }

  /****************************************************************************
   * Remove the median background color
   *
   ***************************************************************************/
  public static BufferedImage removeBackgroundColor(BufferedImage image, boolean binary_image) {
    int mediang[] = new int[image.getHeight() * image.getWidth()];
    int medianr[] = new int[image.getHeight() * image.getWidth()];
    int medianb[] = new int[image.getHeight() * image.getWidth()];

    int co = 0;
    int sumr = 0;
    int sumb = 0;
    int sumg = 0;
    for (int y = 0; y < image.getHeight(); y++) {
      for (int x = 0; x < image.getWidth(); x++) {
        Color c = new Color(image.getRGB(x, y));
        mediang[co] = c.getGreen();
        medianr[co] = c.getRed();
        medianb[co] = c.getBlue();
        sumr += c.getRed();
        sumb += c.getBlue();
        sumg += c.getGreen();
        co++;
      }
    }

    Arrays.sort(mediang);
    Arrays.sort(medianr);
    Arrays.sort(medianb);
    int mg = mediang[mediang.length / 2];
    int mr = medianr[medianr.length / 2];
    int mb = medianb[medianb.length / 2];
    int mdr = sumr / medianr.length;
    int mdg = sumg / mediang.length;
    int mdb = sumb / medianb.length;

  for (int y = 0; y < image.getHeight(); y++) {
      for (int x = 0; x < image.getWidth(); x++) {
        Color c = new Color(image.getRGB(x, y));
        if (c.getRed() - BACKGROUND_THRESHOLD < mdr && mdr < c.getRed() + BACKGROUND_THRESHOLD &&
            c.getGreen() - BACKGROUND_THRESHOLD < mdg && mdg < c.getGreen() + BACKGROUND_THRESHOLD &&
            c.getBlue() - BACKGROUND_THRESHOLD < mdb && mdb < c.getBlue() + BACKGROUND_THRESHOLD) {
          image.setRGB(x, y, Color.BLACK.getRGB());
        } else {
          if (binary_image)
            image.setRGB(x, y, Color.BLUE.getRGB());
        }
      }
    }

    return image;
  }


}
