/***************************************************************************
 * <p>Title: Presence Detection</p>
 * <p>Description: Projects In Computer Science</p>
 * <p>Copyright: Copyright (c) 2007</p>
 * <p>Company: Columbia University; all rights reserved</p>
 *
 * @author Denis Abramov (daa82@columbia.edu)
 * @version 1.0
 *
 * Description:
 * handy class to process image pixels
 ***************************************************************************/
package com.presencedetection.algos;

import java.awt.image.BufferedImage;
import java.awt.Color;

public class ImageData {
  private int _height;
  private int _width;
  private double[][] _data;

  /*****************************************************************
   * Create a blank image of height x width
   *
   *****************************************************************/
  public ImageData(int height, int width) {
    _height = height;
    _width = width;
    _data = new double[height][width];
  }

  /******************************************************************
   * Create a blue image from the buffered image
   *
   ******************************************************************/
   public ImageData(BufferedImage image) {
      this(image.getHeight(), image.getWidth());
      for (int y = 0; y < image.getHeight(); y++) {
        for (int x = 0; x < image.getWidth(); x++) {
          Color c = new Color(image.getRGB(x, y));
          set(x, y, c.getBlue());
        }
      }
  }

  public double get(int x, int y) {
    return _data[y][x];
  }

  public void set(int x, int y, double val) {
    _data[y][x] = val;
  }

  /********************************************************************
   * get the height
   *
   *******************************************************************/
  public int getHeight() {
    return _height;
  }

  /*********************************************************************
   * Get the width
   *
   ********************************************************************/
  public int getWidth() {
    return _width;
  }

  /********************************************************************
   * Subtract an imagedata object from this object
   *
   *******************************************************************/
  public ImageData subtract(ImageData data) {
    ImageData new_data = new ImageData(_height, _width);
    for (int x = 0; x < _width; x++)
      for (int y = 0; y < _height; y++)
        new_data.set(x, y, this.get(x, y) - data.get(x, y));

    return new_data;
  }

  /*******************************************************************
   * Get the absolute value of every pixel (usually performed after
   * the method above)
   *
   ******************************************************************/
  public ImageData abs() {
    ImageData new_data = new ImageData(_height, _width);
    for (int x = 0; x < _width; x++)
      for (int y = 0; y < _height; y++)
        new_data.set(x, y, Math.abs(this.get(x, y)));

    return new_data;
  }


  /*******************************************************************
   * Is presence detected?
   *
   ******************************************************************/
    public boolean isPresenceDetected(int sensitivity) {
       int count = 0;
       for (int y = 0; y < getHeight(); y++) {
         for (int x = 0; x < getWidth(); x++) {
           double rgb = get(x,y);
           if (rgb > 0)
             count++;
         }
       }

       System.out.println("Positive Count: " + count + "/" + (getHeight() * getWidth()));
       if (count > getHeight() * getWidth() / sensitivity)
         return true;
       else
         return false;
     }

}
