/***************************************************************************
 * <p>Title: Presence Detection</p>
 * <p>Description: Projects In Computer Science</p>
 * <p>Copyright: Copyright (c) 2007</p>
 * <p>Company: Columbia University; all rights reserved</p>
 *
 * @author Denis Abramov (daa82@columbia.edu)
 * @version 1.0
 *
 * Description:
 * a simple codec to gain access to the frames AFTER the frame has been
 * processed by the video processor
 * [based on code collected from the web]
 ***************************************************************************/
package com.presencedetection.codecs;

import javax.media.*;
import javax.media.format.*;
import javax.media.util.*;

import java.awt.image.*;
import javax.swing.*;

import com.presencedetection.algos.*;

/*****************************************************************************
 * This codec is called after the image has transitioned
 * in the video
 *
 ****************************************************************************/
public class PostAccessCodec
    extends PreAccessCodec {
  private JLabel _display_panel;
  private final int SHOW_PICTURE_INTERVAL = 20;
  private long _frame_counter;
  private PDAlgoIntf _algo;
  private JLabel _presence_detected_lbl;

  // advertise as supporting all video formats.
  public PostAccessCodec(JLabel panel, JLabel lbl) {
    this();
    _display_panel = panel;
    _presence_detected_lbl = lbl;
  }

  public PostAccessCodec() {
    supportedIns = new Format[] {
        new RGBFormat()
    };
  }

  public synchronized void setAlgo(PDAlgoIntf algo) {
    _algo = algo;
  }

  public synchronized PDAlgoIntf getAlgo() {
    return _algo;
  }

  /**
   * Callback to access individual video frames.
   */
  void accessFrame(Buffer frame) {

    // Capture the frame, process it with our algo and display it on the display panel
    // in our gui
    if (_display_panel != null && _frame_counter % SHOW_PICTURE_INTERVAL == 0) {
      VideoFormat vf = (VideoFormat) frame.getFormat();
      BufferToImage buf2image = new BufferToImage(vf);
      BufferedImage image = (BufferedImage) buf2image.createImage(frame);
      if (image != null) {
        //analyze only every SHOW_PICTURE_INTERVALth frame
        if (_algo != null && _algo.isRunning()) {
          PDResults results = _algo.analyze(image);
          image = results.getImage();
          if (results.isPresenceDetected())
           _presence_detected_lbl.setText("Presence Detected");
         else
           _presence_detected_lbl.setText("");
        }

        ImageIcon icon = new ImageIcon(image);
        _display_panel.setIcon(icon);
        _frame_counter = 0;
      }
    }

    _frame_counter++;
    long t = (long) (frame.getTimeStamp() / 10000000f);

    System.err.println("Post: frame #: " + frame.getSequenceNumber() +
                       ", time: " + ( (float) t) / 100f +
                       ", len: " + frame.getLength());
  }

  public String getName() {
    return "Post-Access Codec";
  }
}
