/***************************************************************************
 * <p>Title: Presence Detection</p>
 * <p>Description: Projects In Computer Science</p>
 * <p>Copyright: Copyright (c) 2007</p>
 * <p>Company: Columbia University; all rights reserved</p>
 *
 * @author Denis Abramov (daa82@columbia.edu)
 * @version 1.0
 *
 * Description:  Dilation image filter
 * The Dilation filter is a morphological filter that changes the shape of
 * objects in an image by dilating (enlarging) the boundaries of bright objects,
 * and reducing the boundaries of dark ones. The dilation filter can be used to
 * increase the size of small bright objects
 *
 ***************************************************************************/

package com.presencedetection.filters;

import java.awt.image.BufferedImage;
import java.util.Arrays;
import com.presencedetection.algos.ImageData;


public class DilationFilter {
  private ImageData _image;
  private ImageData _new_image;
  private int _width;
  private int _height;
  private int _struct[] = { 1, 1, 1,
                            1, 1, 1,
                            1, 1, 1};

  public DilationFilter(ImageData image) {
    _image = image;
    _height = image.getHeight();
    _width = image.getWidth();
    _new_image = new ImageData(image.getHeight(), image.getWidth());
  }

  /***************************************************************************
   * method to apply the filter continuously to an image until
   * the image stabilizes
   *
   **************************************************************************/
  public ImageData applyFilter() {
    boolean changed = true;
    while (changed) {
      changed = false;

      for (int y = 0; y < _height; y++)
        for (int x = 0; x < _width; x++) {
          double new_val = getValue(x, y, 9);
          if (new_val != _new_image.get(x, y))
            changed = true;

          _new_image.set(x, y, new_val);
        }
    }
    return _new_image;
  }

  private int getValue(int start_x, int start_y, int scale) {
    int miny = Math.max(0, start_y - scale / 2);
    int maxy = Math.min(start_y + scale / 2, _height);
    int minx = Math.max(0, start_x - scale / 2);
    int maxx = Math.min(start_x + scale / 2, _width);

    int max = 0;
    int c = 0;
    for (int y = miny; y < maxy; y++) {
      for (int x = minx; x < maxx; x++) {
        double rgb = _image.get(x, y);
        max = Math.max((int) rgb, max);
      }
    }

  return max;

  }
}
