/***************************************************************************
 * <p>Title: Presence Detection</p>
 * <p>Description: Projects In Computer Science</p>
 * <p>Copyright: Copyright (c) 2007</p>
 * <p>Company: Columbia University; all rights reserved</p>
 *
 * @author Denis Abramov (daa82@columbia.edu)
 * @version 1.0
 *
 * Description: A median filter
 * The median filter is a non-linear digital filtering technique, often used to
 * remove noise from images or other signals. The idea is to examine a sample
 * of the input and decide if it is representative of the signal. This is
 * performed using a window consisting of an odd number of samples. The values
 * in the window are sorted into numerical order; the median value,
 * the sample in the center of the window, is selected as the output. The oldest
 * sample is discarded, a new sample acquired, and the calculation repeats.
 *
 ***************************************************************************/

package com.presencedetection.filters;

import java.awt.image.BufferedImage;
import java.util.Arrays;
import com.presencedetection.algos.ImageData;


public class MedianFilter {
  private ImageData _image;
  private ImageData _new_image;
  private int _width;
  private int _height;

  public MedianFilter(ImageData image) {
    _image = image;
    _height = image.getHeight();
    _width = image.getWidth();
    _new_image = new ImageData(image.getHeight(), image.getWidth());
  }

  /***************************************************************************
   * Apply the filter to an image
   *
   **************************************************************************/
  public ImageData applyFilter(int scale) {
    for (int y = 0; y < _height; y++)
      for (int x = 0; x < _width; x++)
        _new_image.set(x, y, getMedianRGB(x, y, scale));

    return _new_image;
  }

  private int getMedianRGB(int start_x, int start_y, int scale) {
    int miny = Math.max(0, start_y - scale / 2);
    int maxy = Math.min(start_y + scale / 2, _height);
    int minx = Math.max(0, start_x - scale / 2);
    int maxx = Math.min(start_x + scale / 2, _width);

    int[] pixels = new int[(maxy - miny) * (maxx - minx)];
    int c = 0;
    for (int y = miny; y < maxy; y++) {
      for (int x = minx; x < maxx; x++) {
        double rgb = _image.get(x, y);
        pixels[c++] = (int) rgb;
      }
    }

   Arrays.sort(pixels);
   if (scale % 2 == 0) {
     int midpoint = (int) pixels.length / 2;
     int midpoint2 = midpoint + 1;
     return (pixels[midpoint] + pixels[midpoint2]) / 2;
   } else {
     return pixels[pixels.length / 2];
   }

  }
}
