/***************************************************************************
 * <p>Title: Presence Detection</p>
 * <p>Description: Projects In Computer Science</p>
 * <p>Copyright: Copyright (c) 2007</p>
 * <p>Company: Columbia University; all rights reserved</p>
 *
 * @author Denis Abramov (daa82@columbia.edu)
 * @version 1.0
 *
 * Description: This is the main GUI class
 ***************************************************************************/
package com.presencedetection.gui;

import java.util.*;
import javax.media.*;
import javax.media.bean.playerbean.*;
import javax.media.format.*;
import javax.media.protocol.*;
import javax.media.util.*;

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;

import com.sun.media.util.*;
import jmapps.jmstudio.*;
import jmapps.ui.*;
import jmapps.util.*;
import javax.media.control.TrackControl;
import com.presencedetection.codecs.PreAccessCodec;
import com.presencedetection.codecs.PostAccessCodec;
import com.borland.jbcl.layout.*;
import java.awt.event.*;
import com.presencedetection.algos.ColorDetectionAlgo;
import com.presencedetection.algos.BackgroundSubtractionAlgo;
import com.presencedetection.algos.PDAlgoIntf;

public class PresenceDetection
    extends JFrame
    implements ControllerListener {

  private String[] _algos = {
      "Color Algo", "Background Algo"};
  private MediaPlayer _player = new MediaPlayer();
  private Processor _proc;
  private Object waitSync = new Object();
  private boolean stateTransitionOK = true;
  private Vector _capture_device_list = new Vector();
  private static JMAppsCfg cfgJMApps = null;
  private CaptureControlsDialog dlgCaptureControls = null;
  private PreAccessCodec _pre_access_codec = new PreAccessCodec();
  JLabel jLabel2 = new JLabel();
  JLabel presence_detected_lbl = new JLabel();
  private PostAccessCodec _post_access_codec = new PostAccessCodec(jLabel2,
      presence_detected_lbl);

  JPanel jPanel2 = new JPanel();
  JComboBox _algobox = new JComboBox();
  JLabel jLabel1 = new JLabel();
  JButton startBtn = new JButton();
  XYLayout xYLayout1 = new XYLayout();
  JPanel image_panel = new JPanel();
  JPanel video_panel = new JPanel();
  XYLayout xYLayout2 = new XYLayout();

  /************************************************************************
   * Setup the GUI
   *
   ***********************************************************************/
  public PresenceDetection() {
    try {
      jbInit();
      checkCaptureDevices();
      captureMedia();
    }
    catch (Exception e) {
      e.printStackTrace();
    }
  }

  /************************************************************************
   * Check to make sure that a capture device is present
   *
   ***********************************************************************/
  private void checkCaptureDevices() {
    _capture_device_list = CaptureDeviceManager.getDeviceList(null);
    if (_capture_device_list == null || _capture_device_list.size() < 1)
      JOptionPane.showMessageDialog(this, "No capture Devices Found!", "ERROR",
                                    JOptionPane.ERROR_MESSAGE);
  }

  /************************************************************************
   * Out of all the streams that are available find the video stream
   *
   ************************************************************************/
  private PushBufferStream getVideoStream(DataSource cdswrapper) throws
      Exception {
    PushBufferStream[] pbs = ( (PushBufferDataSource) cdswrapper).getStreams();
    for (int i = 0; i < pbs.length; i++) {
      if (pbs[i].getFormat() instanceof VideoFormat) {
        return pbs[i];
      }
    }
    throw new Exception("No video format was found!");
  }

  /************************************************************************
   * Create the data sink to the camera video stream
   *
   ***********************************************************************/
  private DataSource createCameraDataSource() throws Exception {
    CaptureDialog dialogCapture;
    DataSource dataSource;
    CaptureDeviceInfo cdi;

    String nameCaptureDeviceAudio = null;
    String nameCaptureDeviceVideo = null;

    dialogCapture = new CaptureDialog(this, cfgJMApps);
    dialogCapture.show();
    if (dialogCapture.getAction() == CaptureDialog.ACTION_CANCEL)
      throw new Exception("Camera Capture has been cancelled!");

    cdi = dialogCapture.getAudioDevice();
    if (cdi != null && dialogCapture.isAudioDeviceUsed())
      nameCaptureDeviceAudio = cdi.getName();
    cdi = dialogCapture.getVideoDevice();
    if (cdi != null && dialogCapture.isVideoDeviceUsed())
      nameCaptureDeviceVideo = cdi.getName();
    dataSource = JMFUtils.createCaptureDataSource(nameCaptureDeviceAudio,
                                                  dialogCapture.getAudioFormat(),
                                                  nameCaptureDeviceVideo,
                                                  dialogCapture.getVideoFormat());

    return dataSource;

  }

  /****************************************************************************
   * Attach the pre-access & post-access codecs to the video stream
   *
   ***************************************************************************/
  private void setupFrameListener(DataSource datasource) throws Exception {
    try {
      _proc = Manager.createProcessor(datasource);
    }
    catch (Exception e) {
      e.printStackTrace();
    }

    _proc.addControllerListener(this);

    // Put the Processor into configured state.
    _proc.configure();
    if (!waitForState(_proc.Configured)) {
      throw new Exception("Failed to configure the processor.");
    }

    // So I can use it as a player.
    _proc.setContentDescriptor(null);

    // Obtain the track controls.
    TrackControl tc[] = _proc.getTrackControls();

    if (tc == null) {
      throw new Exception("Failed to obtain track controls from the processor.");
    }

    // Search for the track control for the video track.
    TrackControl videoTrack = null;

    for (int i = 0; i < tc.length; i++) {
      if (tc[i].getFormat() instanceof VideoFormat) {
        videoTrack = tc[i];
        break;
      }
    }

    if (videoTrack == null) {
      throw new Exception("The input media does not contain a video track.");
    }

    System.err.println("Video format: " + videoTrack.getFormat());

    // Instantiate and set the frame access codec to the data flow path.
    try {
      Codec codec[] = {
          this._pre_access_codec,
          this._post_access_codec};
      videoTrack.setCodecChain(codec);
    }
    catch (UnsupportedPlugInException e) {
      System.err.println("The process does not support effects.");
    }

    // Realize the processor.
    _proc.prefetch();
    if (!waitForState(_proc.Prefetched)) {
      System.err.println("Failed to realize the processor.");
    }
  }

  /********************************************************************
   *
   * Block until the processor has transitioned to the given state.
   * Return false if the transition failed.
   ********************************************************************/
  private boolean waitForState(int state) {
    synchronized (waitSync) {
      try {
        while (_proc.getState() != state && stateTransitionOK)
          waitSync.wait();
      }
      catch (Exception e) {}
    }
    return stateTransitionOK;
  }

  /************************************************************************
   * Start capturing the video from the camera
   *
   ***********************************************************************/
  private void captureMedia() throws Exception {

    DataSource dataSource = createCameraDataSource();
    if (dataSource != null) {
      setupFrameListener(dataSource);

      if (dataSource instanceof CaptureDevice
          && dataSource instanceof PushBufferDataSource) {
        DataSource cdswrapper = new CDSWrapper( (PushBufferDataSource)
                                               dataSource);
        dataSource = cdswrapper;
        try {
          PushBufferStream s = getVideoStream(cdswrapper);

          cdswrapper.connect();
          cdswrapper.start();

        }
        catch (Exception ioe) {
          dataSource = null;
          MessageDialog.createErrorDialog(this,
                                          JMFI18N.getResource(
              "jmstudio.error.captureds"));
        }
      }

      open(dataSource);
      if (dataSource != null) {
        dlgCaptureControls = new CaptureControlsDialog(this, dataSource);
        if (dlgCaptureControls.isEmpty()) {
          dlgCaptureControls = null;
        }
        else {
//        dlgCaptureControls.setVisible ( true );
        }
      }
    }
    else {
      MessageDialog.createErrorDialog(this,
                                      JMFI18N.getResource(
          "jmstudio.error.captureds"));
    }
  }

  /**************************************************************************
   * Startup the video processor and start capturing video
   *
   *************************************************************************/
  public void open(DataSource dataSource) {
    Component cc;
    Component vc;
    if ( (vc = _proc.getVisualComponent()) != null) {
      video_panel.add("Center", vc);
    }

    if ( (cc = _proc.getControlPanelComponent()) != null) {
      video_panel.add("South", cc);
    }

    // Start the processor.
    _proc.start();
    setVisible(true);

    //add to the combo box
    for (int i = 0; i < _algos.length; i++) {
      this._algobox.addItem(_algos[i]);
    }
  }

  /**************************************************************************
   * Init a bunch of GUI stuff. Created with JBuilder Foundation 2005
   *
   *************************************************************************/
  private void jbInit() throws Exception {
    xYLayout2.setWidth(712);
    xYLayout2.setHeight(374);
    this.getContentPane().setLayout(xYLayout2);
    jPanel2.setLayout(xYLayout1);
    jLabel1.setText("Select Algorithm:");
    startBtn.setText("Start Analysis");
    startBtn.addActionListener(new PresenceDetection_startBtn_actionAdapter(this));
    this.setDefaultCloseOperation(EXIT_ON_CLOSE);
    this.setTitle("Presence Detection");
    image_panel.setBorder(BorderFactory.createLineBorder(Color.black));
    video_panel.setBorder(BorderFactory.createLineBorder(Color.black));
    _algobox.setSelectedIndex( -1);
    presence_detected_lbl.setForeground(Color.blue);
    presence_detected_lbl.setBorder(BorderFactory.createRaisedBevelBorder());
    presence_detected_lbl.setHorizontalAlignment(SwingConstants.CENTER);
    presence_detected_lbl.setHorizontalTextPosition(SwingConstants.CENTER);
    presence_detected_lbl.setText("No Presence Detected");
    this.getContentPane().add(jPanel2, new XYConstraints(0, 4, 725, 370));
    jPanel2.add(video_panel, new XYConstraints(6, 0, 351, 275));
    jPanel2.add(image_panel, new XYConstraints(357, 0, 351, 275));
    image_panel.add(jLabel2, null);
    jPanel2.add(presence_detected_lbl, new XYConstraints(246, 282, 180, 24));
    jPanel2.add(_algobox, new XYConstraints(67, 326, 275, 26));
    jPanel2.add(jLabel1, new XYConstraints(67, 307, 267, 21));
    jPanel2.add(startBtn, new XYConstraints(446, 307, 115, 51));
  }

  public static void main(String args[]) {
    PresenceDetection d = new PresenceDetection();
    d.pack();
    d.setVisible(true);
  }

  /*************************************************************************
   * controllerUpdate
   * This will control when the video is started as
   * well as all other events that are pertinent to the video/start/finish
   * and states in between
   *
   * @param controllerEvent ControllerEvent
   *
   **************************************************************************/
  public synchronized void controllerUpdate(ControllerEvent event) {
    if (event instanceof RealizeCompleteEvent ||
        event instanceof PrefetchCompleteEvent ||
        event instanceof ConfigureCompleteEvent) {
      synchronized (waitSync) {
        stateTransitionOK = true;
        waitSync.notifyAll();
      }
    }
    else if (event instanceof ResourceUnavailableEvent) {
      synchronized (waitSync) {
        stateTransitionOK = false;
        waitSync.notifyAll();
      }
    }
    else if (event instanceof EndOfMediaEvent) {
      _proc.close();
      System.exit(0);
    }
  }

  void startBtn_actionPerformed(ActionEvent e) {
    PDAlgoIntf algo = null;

    if (this._algobox.getSelectedItem().equals(_algos[0])) {
      algo = new ColorDetectionAlgo();
    }
    else if (this._algobox.getSelectedItem().equals(_algos[1])) {
      algo = new BackgroundSubtractionAlgo();
    }

    _post_access_codec.setAlgo(algo);
    algo.start();

  }

}

class PresenceDetection_startBtn_actionAdapter
    implements java.awt.event.ActionListener {
  PresenceDetection adaptee;

  PresenceDetection_startBtn_actionAdapter(PresenceDetection adaptee) {
    this.adaptee = adaptee;
  }

  public void actionPerformed(ActionEvent e) {
    adaptee.startBtn_actionPerformed(e);
  }
}
